/*      file     : TrafficData/TAlgoInfo.h
        project  : PAMINA III
        authors  : Marcus Rickert
        RCS      : $Id: TAlgoInfo.h,v 1.4 2005/03/25 12:28:51 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TALGOINFO_H
#define TALGOINFO_H

// ***INCLUDES************************************************************************ 

#include <TParallelListObject.h>
#include <THeap.h>
#include <TDLList.h>
#include <TLocation.h>

// ***FORWARDS************************************************************************

class TAlgoEdge;
class TNode;

// ***DEFINES*************************************************************************

#ifndef INFINITY
#define INFINITY 99999999.0
#endif

#define INVALID_DIST (-1.0)

#define AI_FLAG_NO_SOURCE  1
#define AI_FLAG_NO_SINK    2

// ***TYPES*************************************************************************** 

typedef double DistType;

// ***CLASSES************************************************************************* 

class TAlgoInfo : public TBaseObject, public THeapObject

{
 public:

    TAlgoInfo(void);

    inline DistType
    GetDist(void) const;
    
    inline long
    GetFirstIncidentEdgeIndex(void) const;

    static TAlgoInfo *
    GetInfo(TParallelID ID);
    
    const TLocation &
    GetLocation(void) const;

    inline TNode *
    GetNode(void) const;

    inline int
    GetNrOfIncidentEdges(void) const;

    inline TParallelID
    GetParallelID(void) const;

    inline TAlgoEdge *
    GetPred(void) const;

    bool
    Improved(DistType NewDist, TAlgoEdge *NewPred);

    inline bool
    IsSink(void) const;

    inline bool
    IsSource(void) const;

    void
    Load(TParallelID theParallelID);

    bool
    Load(TNode *theNode);

    void
    Reset(void);

    inline void
    SetDist(DistType theDist);

    inline void
    SetEstimate(DistType theEstimate);

    inline void
    SetFlags(short int theFlags);

    inline void
    SetIncidences(int theFirstIncidentEdge, int theNrOfIncidentEdges);

    inline void
    SetPred(TAlgoEdge *thePred);

    static TClassInfo *ClassInfo;

 private:

    TParallelID  ParallelID;
    TNode       *Node;
    TLocation    Location;
    DistType     Dist;
    TAlgoEdge   *Pred;
    long         Flags;
    long         FirstIncidentEdge;
    long         NrOfIncidentEdges;
    DistType     Estimate;

    virtual int
    HeapCompare(THeapObject *OtherObject, int CompareCode = 0) const;
};

// ***INLINE METHODS****************************************************************** 

#ifndef NO_INLINE
#include "TAlgoInfo.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
