/*      file     : ParData/TAsciiEdgeReader.C
        project  : PAMINA III
        author   : Marcus Rickert
        RCS      : $Id: TAsciiEdgeReader.C,v 1.9 2005/03/29 20:32:32 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include "TDebug.h"

#include "ParToolsTypes.h"

#include "TDatabaseEdge.h"

#include "TAsciiEdgeReader.h"

// ***DEFINES************************************************************************* 

#define EDGES_ASCII_TABLE_NAME "Edges"
#define EDGES_SCAN_PATTERN     "%lld"

// ***METHODS************************************************************************* 

// =================================================================================== 
#define ParentClass TAsciiDataTypeReader
// =================================================================================== 

TAsciiEdgeReader::TAsciiEdgeReader(const char *theBaseFilename,
                                   const char *theIndexScanPattern,
                                   const char *theIndexFileExtension) :

ParentClass(theBaseFilename, EDGES_ASCII_TABLE_NAME, theIndexScanPattern,
            theIndexFileExtension)

{    
}

// =================================================================================== 

void *
TAsciiEdgeReader::CreateObject(const char *Line) const

{
    TDatabaseEdge *Edge = 0;
    static char Format[64] = { 0 };

    if ((Edge = new TDatabaseEdge) == 0)
    {
        Error("cannot create TDatabaseEdge");
        return 0;
    }
    Edge->Level = INVALID;
    if (0 == Format[0]) {
        sprintf(Format, "%s %s %s %%hd %%lf %%hd %%lf %%lf", PARALLEL_ID_PRINTF_FORMAT,
                PARALLEL_ID_PRINTF_FORMAT, PARALLEL_ID_PRINTF_FORMAT);
        Print(DM_FILE, "created scanf format %s", Format);
    }
    if (sscanf(Line, Format, 
               &Edge->ID, &Edge->FromNodeID, &Edge->ToNodeID,
               &Edge->NrOfLanes, &Edge->Length, &Edge->Class,
               &Edge->SpeedLimit, &Edge->FreeSpeed) != 8)
    {
        Error("too few parameters in line '%s'", Line);
        return 0;
    }

    //    Print(Format, Edge->ID, Edge->FromNodeID, Edge->ToNodeID,
    //               Edge->NrOfLanes, Edge->Length, Edge->Class,
    //               Edge->SpeedLimit, Edge->FreeSpeed);
    
    return Edge;
}

// =================================================================================== 

void
TAsciiEdgeReader::DisposeObject(void *Object)

{
    delete (TDatabaseEdge *) Object;
}

// =================================================================================== 
#undef ParentClass
