/*      file     : TrafficData/TAsciiPhaseReader.C
        project  : PAMINA III
        author   : Marcus Rickert
        RCS      : $Id: TAsciiPhaseReader.C,v 1.6 2005/03/29 17:12:48 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <TConfiguration.h>
#include <TDebug.h>

#include <ParToolsTypes.h>

#include "TrafficDataConfig.h"
#include "TPhase.h"
#include "TAsciiPhaseReader.h"

// ***DEFINES************************************************************************* 

#define PHASE_ASCII_TABLE_NAME "Phases"
#define PHASE_SCAN_PATTERN LONG_ID_PRINTF_FORMAT

// ***METHODS************************************************************************* 

// =================================================================================== 
#define ParentClass TAsciiDataTypeReader
// =================================================================================== 

TAsciiPhaseReader::TAsciiPhaseReader(const char *theBaseFilename) :

ParentClass(theBaseFilename, PHASE_ASCII_TABLE_NAME, PHASE_SCAN_PATTERN)

{    
}

// =================================================================================== 

void *
TAsciiPhaseReader::CreateObject(const char *Line) const

{
    TPhase          *Phase = 0;
    TPhaseConstData  ConstData;
    double           GreenPhase, NonGreenPhase;
    double           NonGreenPhaseFactor;
    static char      Format[64] = { 0 };

    NonGreenPhaseFactor = Configuration->GetDouble(INTERSECTION_GROUP,
                                                   INTERSECTION_NAME_NONGREEN_PHASE_FACTOR);
    if (0 == Format[0]) {
        sprintf(Format, "%s %s %%lf %%lf", PARALLEL_ID_PRINTF_FORMAT,
                PARALLEL_ID_PRINTF_FORMAT);
        Print(DM_FILE, "created scanf format %s", Format);
    }
    if (sscanf(Line, Format, &ConstData.NodeID, &ConstData.EdgeID,
               &GreenPhase, &NonGreenPhase) != 4)
    {
        Error("too few parameters in line '%s'", Line);
        return 0;
    }
    ConstData.GreenPhaseLength = (short int) GreenPhase;
    ConstData.NonGreenPhaseLength = (short int) (NonGreenPhaseFactor * NonGreenPhase);
    if ((Phase = new TPhase(ConstData)) == 0)
    {
        Error("cannot create TPhase");
        return 0;
    }
    return Phase;
}

// =================================================================================== 
#undef ParentClass
