/*      file     : TrafficData/TEdgeInfo.C
        project  : PAMINA III
        author   : Marcus Rickert
        RCS      : $Id: TEdgeInfo.C,v 1.4 2005/03/25 12:28:51 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <stdio.h>

#include <TString.h>
#include <TBaseStream.h>

#include <TClassInfo.h>

#include <TTransferInfo.h>
#include <TDatabaseEdge.h>

#include "TEdgeInfo.h"

// ***GLOBAL VARIABLES****************************************************************  

TClassInfo *TEdgeInfo::ClassInfo = 0;

// ***METHODS************************************************************************* 

// =================================================================================== 
#define ParentClass TParallelObject
// =================================================================================== 

TEdgeInfo::TEdgeInfo(TParallelID theParallelID) :

ParentClass(ClassInfo, theParallelID)

{
    Level = INVALID;
    Class = INVALID;
    NrOfLanes = INVALID;
    Length = (double) INVALID;
    SpeedLimit = (double) INVALID;
    FreeSpeed = (double) INVALID;
}

// =================================================================================== 

TEdgeInfo &
TEdgeInfo::operator = (const TDatabaseEdge &DatabaseEdge)

{
    Level = DatabaseEdge.Level;
    Class = DatabaseEdge.Class;
    NrOfLanes = DatabaseEdge.NrOfLanes;
    Length = DatabaseEdge.Length;
    SpeedLimit = DatabaseEdge.SpeedLimit;
    FreeSpeed = DatabaseEdge.FreeSpeed;
    return *this;
}

// =================================================================================== 

bool
TEdgeInfo::Decode(TBaseStream *Stream, TTransferInfo *TransferInfo)

{
    if (!ParentClass::Decode(Stream, TransferInfo))
        return FALSE;
    if (TransferInfo->TransferMode == TM_ConstData)
        return (Stream->Decode(Level) AND
                Stream->Decode(Class) AND
                Stream->Decode(NrOfLanes) AND
                Stream->Decode(Length) AND
                Stream->Decode(SpeedLimit) AND
                Stream->Decode(FreeSpeed));
    else
        return TRUE;
}

// =================================================================================== 

bool
TEdgeInfo::Encode(TBaseStream *Stream, TTransferInfo *TransferInfo) const

{
    if (!ParentClass::Encode(Stream, TransferInfo))
        return FALSE;
    if (TransferInfo->TransferMode == TM_ConstData)
        return (Stream->Encode(Level) AND
                Stream->Encode(Class) AND
                Stream->Encode(NrOfLanes) AND
                Stream->Encode(Length) AND
                Stream->Encode(SpeedLimit) AND
                Stream->Encode(FreeSpeed));
    else
        return TRUE;
}

// =================================================================================== 

TString
TEdgeInfo::GetClassName(void) const

{
    return "TEdgeInfo";
}

// =================================================================================== 

TString
TEdgeInfo::GetInstanceName(void) const

{
    char Line[256];

    sprintf(Line, "Lvl:%d Cls:%d NrLns:%d Len:%.1f SpLmt:%.1f FFSp:%.1f",
            Level, Class, NrOfLanes, Length, SpeedLimit, FreeSpeed);
    return Line;
}

// =================================================================================== 

TEdgeInfo *
TEdgeInfo::GetPtr(TParallelID ID, bool Warn)

{
    return (TEdgeInfo *) ClassInfo->GetObjectPtr(ID, Warn);
}

// =================================================================================== 
#undef ParentClass

