/*      file     : TrafficData/TPhase.C
        project  : PAMINA III
        author   : Marcus Rickert
        RCS      : $Id: TPhase.C,v 1.4 2005/03/25 12:28:52 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************

#include <TParser.h>
#include <TBaseStream.h>
#include <TTimeStep.h>
#include <TTransferInfo.h>

#include "TPhase.h"

// =================================================================================== 

TPhaseVarData::TPhaseVarData(void)

{
    State = InactivePhase;
    NextTimeCheck = INVALID;
}

// =================================================================================== 

TPhaseConstData::TPhaseConstData(void)

{    
    NodeID = INVALID_ID;
    EdgeID = INVALID_ID;
    Offset = 0;
    GreenPhaseLength = 0;
    NonGreenPhaseLength = 0;
}

// =================================================================================== 
#define ParentClass TObject
// ===================================================================================

TPhase::TPhase(void)

{
}

// =================================================================================== 

TPhase::TPhase(const TPhaseConstData &ConstData) :

Const(ConstData)

{
}

// ===================================================================================

bool
TPhase::ComputeOffsets(int RelativePhase)

{
    if (Var.State == InactivePhase)
        if (RelativePhase == 0 AND Const.NonGreenPhaseLength > 0)
        {
            Var.State = NonGreenPhase;
            Var.NextTimeCheck = Const.NonGreenPhaseLength;
        }
        else
        {
            Var.State = GreenPhase;
            Var.NextTimeCheck = Const.GreenPhaseLength;
        }
    return TRUE;
}

// ===================================================================================

bool
TPhase::Decode(TBaseStream *aStream, TTransferInfo *TransferInfo)

{
    switch (TransferInfo->TransferMode)
    {
    case TM_ConstData:
        return (aStream->Decode(Const.NodeID) AND
                aStream->Decode(Const.EdgeID) AND
                aStream->Decode(Const.Offset) AND
                aStream->Decode(Const.GreenPhaseLength) AND
                aStream->Decode(Const.NonGreenPhaseLength));

    case TM_VarData:
        return (aStream->Decode(Var.State) AND
                aStream->Decode(Var.NextTimeCheck));

    default:    
        Warning("invalid TransferMode %d", (int) TransferInfo->TransferMode);
    }
    return TRUE;
}

// ===================================================================================

bool
TPhase::Encode(TBaseStream *aStream, TTransferInfo *TransferInfo) const

{ 
    switch (TransferInfo->TransferMode)
    {
    case TM_ConstData:
        return (aStream->Encode(Const.NodeID) AND
                aStream->Encode(Const.EdgeID) AND
                aStream->Encode(Const.Offset) AND
                aStream->Encode(Const.GreenPhaseLength) AND
                aStream->Encode(Const.NonGreenPhaseLength));

    case TM_VarData:
        return (aStream->Encode(Var.State) AND
                aStream->Encode(Var.NextTimeCheck));

    default:
        Warning("invalid TransferMode %d", (int) TransferInfo->TransferMode);
    }
    return TRUE;            
}

// =================================================================================== 

bool
TPhase::ExecuteSubTimeStep(const TTimeStep &TimeStep)

{
    if (TimeStep >= Var.NextTimeCheck AND 
        Const.NonGreenPhaseLength > 0)
    {
        switch(Var.State)
        {
        case InactivePhase:
            Error("PrepareTimeStep: phase inactive");
            break;
            
        case GreenPhase:
            Var.State = NonGreenPhase;
            Var.NextTimeCheck += Const.NonGreenPhaseLength;
            break;
            
        case NonGreenPhase:
            Var.State = GreenPhase;
            Var.NextTimeCheck += Const.GreenPhaseLength;
            break;
        }
    }
    return TRUE;
}

// ===================================================================================

TString
TPhase::GetClassName(void) const

{
    return "TPhase";
}

// =================================================================================== 
#undef ParentClass

