/*      file     : TrafficData/TPhase.h
        project  : PAMINA III
        author   : Marcus Rickert
        RCS      : $Id: TPhase.h,v 1.5 2005/03/25 12:28:52 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TPHASE_H
#define TPHASE_H

// ***INCLUDES************************************************************************

#include <ParToolsTypes.h>

#include <TBaseObject.h>

// ***FORWARDS************************************************************************

class TParser;
class TBaseStream;
class TTimeStep;
class TTransferInfo;

// ***TYPES*************************************************************************** 

enum TPhaseState

{
    InactivePhase,
    GreenPhase,
    NonGreenPhase
};

// ***CLASSES************************************************************************* 

class TPhaseConstData

{
public:

    TPhaseConstData(void);
    
    TParallelID NodeID;
    TParallelID EdgeID;
    short int   Offset;
    short int   GreenPhaseLength;
    short int   NonGreenPhaseLength;    
};

class  TPhaseVarData

{
public:

    TPhaseVarData(void);
        
    short int State;
    long      NextTimeCheck;
};

class TPhase : public TBaseObject

{
  public:

    TPhase(void);
    TPhase(const TPhaseConstData &theConstData);

    bool
       ComputeOffsets(int RelativePhase);
    virtual bool
       Decode(TBaseStream *aStream, TTransferInfo *TransferInfo);
    virtual bool
       Encode(TBaseStream *aStream, TTransferInfo *TransferInfo) const;
    inline bool
       IsActive(void) const;
    bool
       ExecuteSubTimeStep(const TTimeStep &TimeStep);

    TPhaseConstData Const;

  private:

    virtual TString
       GetClassName(void) const;

    TPhaseVarData Var;
};

// ***INLINE METHODS******************************************************************

#ifndef NO_INLINE
#  include "TPhase.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
