/*      file     : common/Basic.h
        project  : General Utilities
        author   : Marcus Rickert
        RCS      : $Id: Basic.h,v 1.14 2005/03/25 11:04:58 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef BASIC_H
#define BASIC_H

// ***DEFINES*************************************************************************

// SUNPRO CC compiler specific

#ifdef __SUNPRO_CC
#  ifndef NO_BOOL
#    define NO_BOOL
#  endif
#  define INCLUDE_TEMPLATES
#endif

// SUN OS C-library specific

#ifdef SUN4
#  define FREE_TYPE char *
#else
#  define FREE_TYPE void *
#endif

// -----------------------------------------------------------------------------------

#ifndef PI
#define PI 3.1415926
#endif

#define INVALID  (-1)

#ifndef INVALID_ID
#define INVALID_ID (-1)
#endif

#ifndef min
#define min(A,B) (((A) < (B))? (A):(B))
#endif
#ifndef max
#define max(A,B) (((A) > (B))? (A):(B))
#endif

// Oh Pascal !

#define FALSE 0
#define TRUE  1
#define AND   &&
#define OR    ||

// Memory allocation

#define DeleteObject(obj) {if (obj) {delete obj; obj=0;}}
#define DeleteObjectArray(obj) {if (obj) {delete [] obj; obj=0;}}
#define Free(ptr) {if (ptr) {free((FREE_TYPE) (ptr)); ptr=0;}}
#define TrueFalse(Flag) ((Flag)? "TRUE":"FALSE")

// Suppress inlining in '.h' files, but move them to '.C' files
// The GCC on RS6K currently does not work with inlines. Why?

#ifdef NO_INLINE
#define inline
#endif

// Mutable class member support for compilers that don't support 'mutable'
// Very ugly! Don't tell or show anybody!

#ifdef MUTABLE
#define MUTABLE_DECL MUTABLE
#define FriendIfNotMutable(Friend)
#define MutableAccess(Class, Member) Member
#else
#define MUTABLE_DECL
#define FriendIfNotMutable(Friend) friend class Friend;
#define MutableAccess(Class, Member) ((Class *) this)->Member
#endif

// ***TYPES***************************************************************************

#ifdef NO_BOOL
typedef unsigned int bool;
#endif

typedef int                TIndex;
typedef unsigned char      byte;
typedef unsigned short int uword;
typedef bool               TApplyFunction (void *Item, void *Context);

// ===================================================================================

enum TBaseType

{
    BaseTypeChar,
    BaseTypeUChar,
    BaseTypeShort,
    BaseTypeUShort,
    BaseTypeInt,
    BaseTypeUInt,
    BaseTypeLong,
    BaseTypeULong,
    BaseTypeFloat,
    BaseTypeDouble,
    BaseTypeLDouble,
    BaseTypeLLong,
    BaseTypeCount
};

enum 
TBaseOperation

{
    OpMaximum,
    OpMinimum,
    OpSum,
    OpProduct,
    OpLogicalAND,
    OpBitwiseAND,
    OpLogicalOR,
    OpBitwiseOR,
    OpLogicalXOR,
    OpBitwiseXOR,
    BaseOperationCount
};

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
