/*      file     : common/TABTreeElement.h
        project  : General Toolbox
        author   : Marcus Rickert
        RCS      : $Id: TABTreeElement.h,v 1.5 2005/03/25 11:04:59 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TABTREEELEMENT_H
#define TABTREEELEMENT_H

// ***INCLUDES************************************************************************ 

#include <TBaseObject.h>

// ***FORWARDS************************************************************************

class TABTreePrimitive;
class TSelectorPrimitive;

// ***DEFINES************************************************************************* 

#define TAB_A 2
#define TAB_B 4
#define TAB_SPLIT ((int) ((TAB_B+1)/2))

// ***CLASSES************************************************************************* 

class TABTreeElement : public TBaseObject /* see comment in TID2Table.h */

{
 public:

    TABTreeElement(TABTreeElement *FatherPar, bool IsLeafPar = TRUE);
    ~TABTreeElement(void);

    bool
        Apply(TApplyFunction Function, void *Context);
    bool
        CanDelete(void);
    bool
        CanInsert(void);
    bool
        Check(TABTreeElement *TrueFather);
    void
        Delete(int DeletionPoint);

    void
    DeleteObjects(void);
    
    bool
        DeletePossible(void);
    virtual TString
        GetClassName(void) const;
    virtual TString
        GetInstanceName(void) const;
    inline TBaseObject *
        GetLeftMostItem(void);
    TBaseObject *
        GetCompareItem(int Son);
    inline int
        GetElementsUsed(void);
    inline TABTreeElement *
        GetFather(void);
    TABTreeElement *
        GetLeftNeighbour(void);
    inline TABTreeElement *
        GetNext(void) const;
    inline TABTreeElement *
        GetPrev(void) const;
    TABTreeElement *
        GetRightNeighbour(void);
    inline TABTreeElement *
        GetSon(int SonNumber);
    int
        GetSonNumber(TABTreeElement *Son);
    void
        Insert(int InsertionPoint, TBaseObject *Data);
    inline bool
        InsertPossible(void);
    inline bool
        IsLeaf(void);
    void
        Merge(TABTreeElement *RightNeighbour);
    TABTreeElement *
        StealLeftmostSon(void);
    TABTreeElement *
        StealRightmostSon(void);
    void
        TraceLeftmostItem(void);
    void
        Remove(int SonNr);
    void
        Reset(void);
    void
        ResetLeaf(void);
    void
        RetrieveLeftmostItem(void);
    inline void
        SetFather(TABTreeElement *AFather);
    inline void
        SetLeaf(bool IsLeaf = TRUE);
    bool
        SetLeftMostItem(TABTreeElement *Son);   
    void
        Split(TABTreeElement *NewElement);
    inline bool
        TooFewSons(void);
    
 private:

    TBaseObject    *LeftMostItem;
    int             ElementsUsed;
    TABTreeElement *Father;
    TABTreeElement *Next;
    TABTreeElement *Prev;
    TABTreeElement *References[TAB_B];
    bool            Leaf;
};

// ***INLINE METHODS******************************************************************

#ifndef NO_INLINE
#  include "TABTreeElement.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif

