/*      file     : common/TAlarm.C
        project  : General Toolbox
        author   : Marcus Rickert
        RCS      : $Id: TAlarm.C,v 1.6 2005/03/25 11:04:59 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <unistd.h>

#include "TDebug.h"
#include "TString.h"
#include "TSignal.h"
#include "TAlarm.h"

// ***METHODS************************************************************************* 

#ifdef OLD_ALARM
TAlarm::TAlarm(TSignal *theSignalHandler)
#else
TAlarm::TAlarm(TSignal *)
#endif

{
#ifdef OLD_ALARM
    SignalHandler = theSignalHandler;
    Active = FALSE;
#endif
}

/* =================================================================================== */

TAlarm::~TAlarm(void)

{
    Deactivate();
}

/* =================================================================================== */

bool
TAlarm::Activate(long Interval)

{
#ifdef OLD_ALARM
    struct itimerval Timer;
    
    Timer.it_interval.tv_sec = Interval / 1000L;
    Timer.it_interval.tv_usec = 1000L * (Interval % 1000L);
    Timer.it_value = Timer.it_interval;
    if (Active)
        setitimer(ITIMER_REAL, &Timer, 0);
    else
    {
        if (!SignalHandler->CatchSignal(SIGALRM))
            return FALSE;
        setitimer(ITIMER_REAL, &Timer, &OldTimer);
        Active = TRUE;
    }
#else
    Timer.tv_sec = (int) (Interval / 1000L);
    Timer.tv_usec = 1000L * (Interval % 1000L);    
#endif
    return TRUE;
}

/* =================================================================================== */

void
TAlarm::Deactivate(void)

{
    debug(Print(DM_ALARM, "Deactivate"));
#ifdef OLD_ALARM
    if (Active)
       {
       // force value to be 0 (for LINUX), otherwise a leftover timer signal
       // causes the system to stop with message "Alarm Clock"
    
       OldTimer.it_value.tv_sec = 0;
       OldTimer.it_value.tv_usec = 0;
        
       setitimer(ITIMER_REAL, &OldTimer, 0);
       SignalHandler->RestoreSignal(SIGALRM);
       Active = FALSE;
    }
#endif
}

/* =================================================================================== */

TString
TAlarm::GetClassName(void) const

{
    return "TAlarm";
}

/* =================================================================================== */

void
TAlarm::Pause(void)

{
#ifdef OLD_ALARM
    pause();
#else
    select(0, 0, 0, 0, &Timer);
#endif
}

// =================================================================================== 
