/*      file     : common/TArray.t.C
        project  : General Toolbox
        author   : Marcus Rickert
        RCS      : $Id: TArray.t.C,v 1.12 2005/03/25 11:04:59 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************

#include "gcc_pragma_interface.h"

//#if !defined(__GNUC__)
#  include <memory.h>
//#endif

// ***METHODS************************************************************************* 

template <class T>
TArray<T>::TArray(char *theName) :

#ifdef DARRAY_VAR_ITEMSIZE
TArrayPrimitive(sizeof(T), TARRAY_DEFAULT_INCREMENT, theName)
#else
TArrayPrimitive(TARRAY_DEFAULT_INCREMENT, theName)
#endif

{
}

/* =================================================================================== */

template <class T>

TArray<T>::TArray(int theIncrement, char *theName) :

#ifdef DARRAY_VAR_ITEMSIZE
TArrayPrimitive(sizeof(T), theIncrement, theName)
#else
TArrayPrimitive(theIncrement, theName)
#endif

{
}

/* =================================================================================== */

template <class T>
bool
TArray<T>::operator=(const TArray<T> &anArray)

{
    if (!Resize(anArray.GetNrOfItems()))
        return FALSE;
    if (anArray.GetPhysicalNrOfItems())
        memcpy((void *) Items, (void *) anArray.GetItemPtr(), GetItemSize()*GetNrOfItems());
    return TRUE;
}

/* =================================================================================== */

template <class T>
bool
TArray<T>::operator += (const TArray<T> &anArray)

{
    if (anArray.GetNrOfItems() > 0)
    {
        long OldNrOfItems;

        OldNrOfItems = GetNrOfItems();
        if (!Resize(GetNrOfItems() + anArray.GetNrOfItems()))
            return FALSE;
        memcpy((T *) & Get(OldNrOfItems), (void *) anArray.GetItemPtr(),
               GetItemSize() * anArray.GetNrOfItems());
    }
    return TRUE;
}

/* =================================================================================== */

template <class T>
inline bool
TArray<T>::Add(const T &Item)

{
    return TArrayPrimitive::_Add((void *) &Item);
}

/* =================================================================================== */

template <class T>
const T &
TArray<T>::Get(long Index) const

{
#ifdef DRANGECHECK
    if (Index < 0 OR Index >= GetNrOfItems())
        Error("Get[%ld]", Index);
#endif
    return ((T *) Items) [Index];
}

/* =================================================================================== */

template <class T>
T &
TArray<T>::Get(long Index)

{
#ifdef DRANGECHECK
    if (Index < 0 OR Index >= GetNrOfItems())
        Error("Get[%ld]", Index);
#endif
    return ((T *) Items) [Index];
}

/* =================================================================================== */

template <class T>
TString
TArray<T>::GetClassName(void) const

{
#ifdef DARRAYNAME
    if (!Name.IsEmpty() > 0)
        return Name;
    else
        return "TArray";
#else
    return "TArray";
#endif
}

/* =================================================================================== */

template <class T>
T *
TArray<T>::GetItemPtr(void)

{
    return (T *) Items;
}

/* =================================================================================== */

template <class T>
const T *
TArray<T>::GetItemPtr(void) const

{
    return (T *) Items;
}

/* =================================================================================== */
#ifndef DARRAY_VAR_ITEMSIZE
template <class T>
long
TArray<T>::GetItemSize(void) const

{
    return sizeof(T);
}
#endif
/* =================================================================================== */

template <class T>
bool
TArray<T>::InsertAt(long Index, T& Item)

{
    return TArrayPrimitive::_InsertAt(Index, (void *) &Item);
}

/* =================================================================================== */

template <class T>
void
TArray<T>::Set(long Index, T& Item)

{
    ((T *)Items)[Index] = Item;
}

// =================================================================================== 
