/*      file     : common/TBaseObject.C
        project  : General Toolbox
        author   : Marcus Rickert
        RCS      : $Id: TBaseObject.C,v 1.6 2005/03/25 11:04:59 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************

#define _NO_PROTO
#include <stdio.h>

#include <Basic.h>
#include <TDebug.h>
#include <TString.h>
#include <TEnvironment.h>
#include <TBaseObject.h>

// ***METHODS*************************************************************************

#ifdef NO_INLINE
#  include <TBaseObject.i.C>
#endif

// ===================================================================================

int
TBaseObject::Compare(TBaseObject *, int) const

{
    NotOverloaded("Compare");
    return 0;
}

// ===================================================================================

TBaseObject *
TBaseObject::Copy(void) const

{
    Error("call to method Copy");
    return 0;
}

// ===================================================================================

void
TBaseObject::Error(const char *Format, ...) const

{
    va_list argptr;
    char    line[1024];

    va_start(argptr, Format);
    vsprintf(line, Format, argptr);
    if (Environment)
        Environment->PrintString("ERROR in %s(%s): %s", (char *) GetClassName(), 
                                 (char *) GetInstanceName(), line);
    else
        fprintf(stderr, "ERROR in %s(%s): %s\n", (char *) GetClassName(), 
                (char *) GetInstanceName(), line);
    va_end(argptr);
}

// ===================================================================================

TString
TBaseObject::GetClassName(void) const

{
    return "TBaseObject";
}

// ===================================================================================

TString
TBaseObject::GetInstanceName(void) const

{
    if (Failed())
        return "failed";
    else
        return "";
}

// ===================================================================================

TBaseObject *
TBaseObject::GetInstancePtr(void)

{
    return this;
}

// ===================================================================================

TString
TBaseObject::GetObjectName(void) const

{
    char line[1024];

    sprintf(line, "%s(%s)", (char *) GetClassName(), (char *) GetInstanceName());
    return line;
}

// ===================================================================================

void
TBaseObject::NotOverloaded(const char *MethodName) const

{
    Error("Method %s is not overloaded!", MethodName);
}

// ===================================================================================

void
TBaseObject::Print(const char *Format, ...) const

{
    va_list argptr;
    char    line[2048];

    va_start(argptr, Format);
    vsprintf(line, Format, argptr);
    if (Environment)
        Environment->PrintString("%s(%s): %s", (char *) GetClassName(), 
                                 (char *) GetInstanceName(), line);
    else
        fprintf(stderr, "%s\n", line);
    va_end(argptr);
}

// ===================================================================================
#ifdef TDEBUG
void
TBaseObject::Print(unsigned long Mask, const char *Format, ...) const

{
    va_list argptr;
    char    line[2048];

    if (Debug.GetMask() & Mask)
    {
        va_start(argptr, Format);
        vsprintf(line, Format, argptr);
        if (Environment)
            Environment->PrintString("%s(%s): %s", (char *) GetClassName(), 
                                     (char *) GetInstanceName(), line);
        else
            fprintf(stderr, "%s\n", line);          
        va_end(argptr);
    }
}
#endif
// ===================================================================================

void
TBaseObject::Warning(const char *Format, ...) const

{
    va_list argptr;
    char    line[1024];

    va_start(argptr, Format);
    vsprintf(line, Format, argptr);
    if (Environment)
        Environment->PrintString("%s(%s): WARNING: %s", (char *) GetClassName(), 
                                 (char *) GetInstanceName(), line);
    else
        fprintf(stderr, "%s(%s): WARNING: %s\n", (char *) GetClassName(), 
                (char *) GetInstanceName(), line);
    va_end(argptr);
}

// ===================================================================================
#undef FatherObject






