/*      file     : common/TBaseStream.h
        project  : General Toolbox
        author   : Marcus Rickert
        RCS      : $Id: TBaseStream.h,v 1.7 2005/03/25 11:04:59 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TBASESTREAM_H
#define TBASESTREAM_H

// ***INCLUDES************************************************************************ 

#include <CommonTypes.h>
#include <TBaseObject.h>

// ***CLASSES************************************************************************* 

/** Base class of all classes representing a sequential storage. 
 */

class TBaseStream : public TBaseObject

{
public:
    
#include "TBaseStream.Headers.h"

    virtual bool
    TBaseStream::Decode(LongID &Data, long Count = 1, long Stride = 1);

    virtual bool
    TBaseStream::Encode(const LongID &Data, long Count = 1, long Stride = 1) const;


    /** Decode a boolean value as 'unsigned int'. */
#ifndef NO_BOOL
    bool
        DecodeBool(bool &Data, long Count = 1, long Stride = 1);
#endif
    /** Decode an Ascii string from the message. 
      @param aRawString a reference to a pointer which can hold the string. No range
      checking is performed. */
    bool
        DecodeStr(char **aRawString);
    /** Decode an Ascii string from the message. 
      @param aRawString a pointer to a pointer that will hold the string. Memory is 
      allocated using malloc. */
    bool
        DecodeStr(char *aRawString);
#ifndef NO_BOOL
    /** Encode a boolean value as 'unsigned int'. */
    bool
        EncodeBool(const bool &Data, long Count = 1, long Stride = 1) const;
#endif
    /** Encode an event ID into the current message. 
     */
    virtual bool
        Event(TEventID theEventID) const;
    bool
        EncodeStr(const char *aRawString) const;

    virtual bool
    IsBinary(void) const;

    virtual bool
    Write(const char *Format, ...);
};

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
