/*      file     : common/TConfig.C
        project  : General Toolbox
        author   : Marcus Rickert
        RCS      : $Id: TConfig.C,v 1.11 2005/03/25 11:04:59 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <string.h>

#include <TParser.h>
#include <TConfig.h>

// ***DEFINES************************************************************************* 

#define VALUE_DELIMITER '='
#define SUBVALUE_DELIMITER ':'

// ***CONSTANTS*********************************************************************** 

char *Config_ConfigEntries[] = 

{ 
    "Include"
};

enum TSimulationMasterConfigID

{
    Config_Include,
    
    Config_ConfigIDCount
};

// ***METHODS************************************************************************* 

// =================================================================================== 
#define ParentClass TBaseObject
// =================================================================================== 

TConfig::TConfig(void)

{
    Tokens = 0;
    TokenCount = 0;
    SignatureFile = 0;
    CurrentParser = 0;
    Register(this);
}

// =================================================================================== 

TConfig::~TConfig(void)

{
    DisposeTokens();
}

// =================================================================================== 

void
TConfig::BuildTokens(void)

{
    TConfigHandlerEnum CHandler;
    int                i;

    if (Tokens != 0)
        return;
    TokenCount = 0;
    CHandler.WalksForwardThrough(Handlers);
    while (CHandler())
    {
        CHandler->ConfigIDBase = TokenCount;
        TokenCount += CHandler->GetConfigEntryCount();
        CHandler++;
    }
    Tokens = new char *[TokenCount + 1];
    for (i = 0; i < TokenCount; i++)
        Tokens[i] = 0;
    i = 0;
    CHandler.WalksForwardThrough(Handlers);
    while (CHandler())
    {
        CHandler->RegisterTokens(i, Tokens);
        CHandler++;
    }
    if (i != TokenCount)
        Warning("token number mismatch: %d defined, but %d expected!", i, TokenCount);
}

// =================================================================================== 

bool
TConfig::CheckReadAccess(const char *Filename) const

{
    FILE *file;

    if ((file = fopen(Filename, "r")) == 0)
        return FALSE;
    fclose(file);
    return TRUE;
}

/* =================================================================================== */

bool
TConfig::CommandLineConfigItem(const char *Option)

{
    TConfigHandlerEnum CHandler;

    char  Identifier[256];
    char  Value[256];
    char *Position;
    int   i;
    int   Index;

    if ((Position = strchr(Option, VALUE_DELIMITER)) == 0)
    {
        Error("cannot find identifier in option '%s'", Option);
        return FALSE;
    }
    strncpy(Identifier, Option, Position-Option);
    Identifier[Position-Option] = 0;
    strcpy(Value, Position + 1);
    Index = 0;
    CHandler.WalksForwardThrough(Handlers);
    while (CHandler())
    {
        for (i = 0; i < CHandler->GetConfigEntryCount(); i++)
        {
            if (StrCmp(Tokens[Index], Identifier) == 0)
                return HandleItem(CHandler, i, Value);
            Index++;
        }
        CHandler++;             
    }
    Error("cannot find command line identifier '%s'", Identifier);
    return FALSE;
}

// =================================================================================== 

void
TConfig::DisposeTokens(void)

{
    DeleteObjectArray(Tokens);
    TokenCount = 0;
}

// =================================================================================== 

TString
TConfig::GetClassName(void) const

{
    return "TConfig";
}

/* =================================================================================== */

int
TConfig::GetConfigEntryCount(void) const

{
    return Config_ConfigIDCount;
}

// =================================================================================== 

TString
TConfig::GetInstanceName(void) const

{
    if (CurrentParser != 0)
        return (char *) CurrentParser->GetInstanceName();
    else
        return "no file";
}

/* =================================================================================== */

bool
TConfig::HandleConfigItem(int ID, const char *, const char *Value)

{
    switch (ID)
    {
    case Config_Include:
        return Load(Value, TRUE);
    }
    return TRUE;
}

// =================================================================================== 

bool
TConfig::HandleItem(TConfigHandler *Handler, int ID, const char *Value)

{
    return Handler->HandleConfigItem(ID, 0, Value);
}

// =================================================================================== 

bool
TConfig::Load(const char *ConfigName, bool MustExist)

{
    TParser *Parser;
    TParser *PreviousParser;

    if (!CheckReadAccess(ConfigName))
    {
        if (MustExist)
            Error("config file '%s' not found", ConfigName);
        else
            Warning("cannot read config file '%s'", ConfigName);
        return !MustExist;
    }
    BuildTokens();
    PreviousParser = CurrentParser;
    CurrentParser = Parser = new TParser(TokenCount, Tokens, "= ");
    if (!(Parser->Open(ConfigName) AND
          Read(Parser)))
        return FALSE;
    DeleteObject(Parser);
    CurrentParser = PreviousParser;
    return TRUE;
}

// =================================================================================== 

bool
TConfig::Read(TParser *Parser)

{
    int                 TokenID;
    const char         *Token = 0;
    const char         *Value = 0;
    TConfigHandlerEnum  CHandler;

    if ((Token = Parser->ReadToken()) == 0)
        return FALSE;
    while (!Parser->EndOfFile())
    {   
        if ((TokenID = Parser->GetTokenID()) == -1)
            Warning("illegal token '%s' not handled", (Token == 0)? "(null)":Token);
        else
        {
            Value = Parser->ReadToken();
            if (Value)
            {
                CHandler.WalksForwardThrough(Handlers);
                while (CHandler())
                {
                    if (TokenID >= CHandler->ConfigIDBase AND
                        TokenID < CHandler->ConfigIDBase + CHandler->GetConfigEntryCount())
                    {                   
                        /*
                           Print("HandleConfigItem Identifier:'%s' Value:'%s'",
                           (Identifier==0)? "null":Identifier,
                           (Value==0)? "null":Value); */                        
                        if (!HandleItem(CHandler, TokenID - CHandler->ConfigIDBase, Value))
                            return FALSE;
                        break;
                    }
                    CHandler++;
                }
            }
        }
        Token = Parser->ReadToken();
    }
    return TRUE;
}

// =================================================================================== 

void
TConfig::Register(TConfigHandler *Handler)

{
    DisposeTokens();
    Handlers.Append(Handler);
    Handler->SetConfig(this);
}

// =================================================================================== 

void
TConfig::RegisterTokens(int &Index, char **NewTokens)

{
    RegisterTokenArray(Index, NewTokens, Config_ConfigIDCount, Config_ConfigEntries);
}

// =================================================================================== 

void
TConfig::SignatureHeader(const char *header) const

{
    if (SignatureFile)
    {
        fprintf(SignatureFile, "%c==================================================\n",
                COMMENT_CHAR);
        fprintf(SignatureFile, "%cConfiguration Object: %s\n", COMMENT_CHAR, header);
        fprintf(SignatureFile, "%c--------------------------------------------------\n",
                COMMENT_CHAR);
    }
    else
        Error("no signature file active, cannot write '%s'", header);
}

// =================================================================================== 

void
TConfig::_SignatureLine(const char *line) const

{
    if (SignatureFile)
        fprintf(SignatureFile, "%c%s\n", COMMENT_CHAR, line);
    else
        Error("no signature file active, cannot write '%s'", line);
}

// =================================================================================== 

void
TConfig::WriteCompleteSignature(FILE *file) const

{
    TConfigHandlerEnum CHandler;

    MutableAccess(TConfig, SignatureFile) = file;
    CHandler.WalksForwardThrough(Handlers);
    while (CHandler())
    {
        CHandler->WriteSignature();
        CHandler++;             
    }
    fprintf(file, "%c==END OF CONFIG PORTION===========================\n",
            COMMENT_CHAR);
    fflush(file);
    MutableAccess(TConfig, SignatureFile) = 0;
}

// =================================================================================== 

void
TConfig::WriteSignature(void) const

{
//    SignatureHeader(GetClassName());
}

// =================================================================================== 
#undef ParentClass
