/*      file     : common/TString.C
        project  : General Toolbox
        author   : Marcus Rickert
        RCS      : $Id: TString.C,v 1.17 2005/03/25 11:05:00 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************

#include <cassert>

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <Basic.h>

#include "TDebug.h"
#include "TBaseStream.h"

#include "TString.h"

/* ***FUNCTIONS*********************************************************************** */

#ifdef NO_INLINE
#include "TString.i.C"
#endif

// ===================================================================================
#ifdef NO_STRDUP
char *
strdup(const char *aString)

{
    char *buffer;

    if (aString == 0)
    {
        fprintf(stderr, "strdup: empty string\n");
        return 0;
    }
    if ((buffer = (char *) malloc(strlen(aString) + 1)) == 0)
    {
        fprintf(stderr, "strdup: out of memory\n");
        return 0;
    }
    strcpy(buffer, aString);
    return buffer;
}

#endif
// ===================================================================================

TString
CombinePaths(const char *Path1, const char *Path2)

{
    char Buffer[1024];
    int  Length1;

    if (Path2[0] == PATH_SEPERATOR OR Path1[0] == 0)
        return Path2;
    Buffer[0] = 0;
    Length1 = (int) strlen(Path1);
    strcat(Buffer, Path1);    
    if (Buffer[Length1 - 1] != PATH_SEPERATOR)
    {
        Buffer[Length1] = PATH_SEPERATOR;
        Buffer[Length1 + 1] = 0;
    }
    strcat(Buffer, Path2);
    return Buffer;
}

// ===================================================================================

TString 
GetFilenamePart(const char *aString)

{
    long i;

    i = (long) strlen(aString) - 1;
    while (i >= 0 AND aString[i] != PATH_SEPERATOR)
        i--;
    i++;
    return aString + i;
}

// ===================================================================================

TString 
GetPathPart(const char *aString)

{
    char    *Pos;
    char     Buffer[1024];
    
    strcpy(Buffer, aString);
    Pos = strrchr(Buffer, PATH_SEPERATOR);
    if (Pos)
    {
        *Pos = 0;
        return Buffer;
    }
    else
        return "";
}

// ===================================================================================

int
StrCmp(const char *S1, const char *S2, int CompareCode)

{
    assert(S1 != 0);
    assert(S2 != 0);

    int i = 0;

    if (CompareCode & STRCMP_IGNORE_CASE)
        while (S1[i] AND S2[i])
        {
            if (toupper(S1[i]) > toupper(S2[i]))
                return 1;
            else
                if (toupper(S1[i]) < toupper(S2[i]))
                    return -1;
            i++;
        }
    else
        while (S1[i] AND S2[i])
        {
            if (S1[i] > S2[i])
                return 1;
            else
                if (S1[i] < S2[i])
                    return -1;
            i++;
        }       
    if (S1[i])
        return ((CompareCode & STRCMP_PREFIX_MASK) == STRCMP_S2_PREFIX)? 0:1;
    else
        if (S2[i])
            return ((CompareCode & STRCMP_PREFIX_MASK) == STRCMP_S1_PREFIX)? 0:-1;
        else
            return 0;
}

// ===================================================================================

TString
SubString(const char *aString, int First, int Last)

{
    TString Buffer = aString;

    if (First < 0)
        First = 0;
    if (Last < 0)
        Last = Buffer.Length() + Last - 1;
    if (Last >= Buffer.Length())
        Last = Buffer.Length() - 1;
    if (Last < First)
        return "";
    Buffer.GetPtr()[Last + 1] = 0;
    return Buffer.GetPtr() + First;
}

/* ***METHODS************************************************************************* */

TString::TString(char aChar)

{
    char Buffer[2] = { aChar, 0 };

    _Init();
    SetString(Buffer);
}

/* =================================================================================== */

TString::TString(const TString &aString)

{
    _Init();
    SetString(aString);
}

/* =================================================================================== */

TString::TString(const char *aString)

{
    _Init();
    SetString(aString);
}

/* =================================================================================== */

TString::~TString(void)

{
    FreeString();
}

/* =================================================================================== */

int
TString::operator == (const TString &aTString) const

{
    CheckEmptyS("==", 0);
    return StrCmp(S, aTString()) == 0;
}

/* =================================================================================== */

int
TString::operator == (const char *aString) const

{
    CheckEmptyS("==", 0);
    return StrCmp(S, aString) == 0;
}

/* =================================================================================== */

int
TString::operator != (const TString &aTString) const

{
    CheckEmptyS("!=", 0);
    return StrCmp(S, aTString()) != 0;
}

/* =================================================================================== */

int
TString::operator != (const char *aString) const

{
    CheckEmptyS("!=", 0);
    return StrCmp(S, aString) != 0;
}

/* =================================================================================== */

TString
operator+(const TString &S1, const TString &S2)

{
    char *Buffer;
    TString NewString;

    Buffer = (char *) malloc(S1.Length() + S2.Length() + 1);
    Buffer[0] = 0;
    strcpy(Buffer, S1);
    strcat(Buffer, S2);
    NewString = Buffer;
    free(Buffer);
    return NewString;
}

/* =================================================================================== */

void
TString::operator = (const char *AString)

{
    FreeString();
    SetString(AString);
}

/* =================================================================================== */

const TString &
TString::operator = (const TString &aString)

{
    if (&aString != this)
    {
        FreeString();
        SetString(aString());
    }
    return *this;
}

/* =================================================================================== */

char *
TString::operator()(void) const

{
    assert(S != 0);
    //    CheckEmptyS("operator ()", 0);
    return S;
}

/* =================================================================================== */

const char *
TString::GetPtr(void) const

{
    assert(S != 0);
    //    CheckEmptyS("GetPtr const", 0);
    return S;
}

/* =================================================================================== */

char *
TString::GetPtr(void)

{
    assert(S != 0);
    //    CheckEmptyS("GetPtr", 0);
    return S;
}

/* =================================================================================== */

char 
TString::operator[](int i) const

{
    CheckEmptyS("operator []", '?');
    return S[i];
}

/* =================================================================================== */

TString::operator char * ()

{
    CheckEmptyS("operator char *", _NullString);
    return S;
}

/* =================================================================================== */

TString::operator const char * () const

{
    CheckEmptyS("operator const char *", _NullString);
    return S;
}

/* =================================================================================== */

int
TString::Compare(TBaseObject *OtherObject, int CompareCode) const

{
    TString *OtherString = (TString *) OtherObject;

    CheckEmptyS("Compare", 0);

    // Print("compare '%s' to '%s'", S, OtherString->GetPtr());

    return StrCmp(S, *OtherString, CompareCode);
}

/* =================================================================================== */

void
TString::Consume(char *aString)

{
    FreeString();
    S = aString;
    _Length = (int) strlen(S);
}

/* =================================================================================== */

TBaseObject *
TString::Copy(void) const

{
    return new TString(S);
}

// =================================================================================== 

bool
TString::Decode(TBaseStream &aStream)

{
    char *aRawString = 0;

    if (!aStream.DecodeStr(&aRawString))
        return FALSE;
    Consume(aRawString);
    return TRUE;
}

// =================================================================================== 

bool
TString::Encode(TBaseStream &aStream) const

{
    return aStream.EncodeStr(GetPtr());
}

/* =================================================================================== */

void
TString::FreeString(void)

{
    if (S != 0)
    {
        free(S);
        S = 0;
    }
}

/* =================================================================================== */

TString
TString::GetClassName(void) const

{
    return "TString";
}

/* =================================================================================== */

TString
TString::GetInstanceName(void) const

{
    if (S)
        return S;
    else
        return _NullString;
}

/* =================================================================================== */

void
TString::_Init(void)

{
    S = 0;
    _Length = -1;
}

/* =================================================================================== */

void
TString::SetString(const char *AString)

{
    if (AString)
    {
        _Length = (int) strlen(AString);
        S = (char *) malloc(_Length + 1);
        strcpy(S, AString);
    }
}

// ===================================================================================

void
TString::ToLowerCase(void)

{
    CheckEmptyS("ToLowerCase", );
    for (int i = 0; i < _Length; i++)
        S[i] = tolower(S[i]);
}

// ===================================================================================

void
TString::ToUpperCase(void)

{
    CheckEmptyS("ToLowerCase", );
    for (int i = 0; i < _Length; i++)
        S[i] = toupper(S[i]);
}

// ===================================================================================

