/*      file     : mpi/TMPIMessageControl.C
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TMPIMessage.C,v 1.8 2005/05/01 14:57:58 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <stdio.h>
#include <string.h>

#include <TDestination.h>
#include "Mpi.h"
#include "TMPIBuffer.h"
#include "TMPIMessage.h"

// ***DEFINES************************************************************************* 

#define MPI_DEFAULT_MSG_TYPE 1

// ***METHODS************************************************************************* 

#include "TMPIMessage.Bodies.C"

// =================================================================================== 
#define ParentClass TBaseMessage
// =================================================================================== 

TMPIMessage::TMPIMessage(TCommunicationControl *theControl, TMPIBuffer *theBuffer, 
                         TCPNID theSender) :

ParentClass(theControl, theSender)

{
    _Init();
    Buffer = theBuffer;
    Buffer->Activate(this);
}

// =================================================================================== 

TMPIMessage::TMPIMessage(TCommunicationControl *theControl, TMPIBuffer *theBuffer, 
                         TMessageID theID, TCPNID theSender, 
                         TDestination *theDestination,
                         TMessageTag theTag, TMessageType theType,
                         TMessageFlags theFlags) :

                         TBaseMessage(theControl, theID, theSender, theDestination, 
                                      theTag, theType, theFlags)

{
    _Init();
    Buffer = theBuffer;
    Buffer->Activate(this);
}

// =================================================================================== 

TMPIMessage::~TMPIMessage(void)

{
    if (Buffer)
        Buffer->Deactivate();
}

// =================================================================================== 

bool
TMPIMessage::BeginSend(void)

{
    return ParentClass::BeginSend();
}

// =================================================================================== 

bool
TMPIMessage::EndSend(void)

{
    const TCPNID *Recipients;
    int           i;

    if (!ParentClass::EndSend())
        return FALSE;
    switch(GetDestination()->GetRecipient())
    {
    case DEST_LIST:
    case DEST_ALL:
    case DEST_ALL_BUT_MYSELF:
        if ((Recipients = GetDestination()->GetRecipients()) == 0)
        {
            Error("EndSend: cannot retrieve recipients");
            return FALSE;
        }
        for (i = 0; i < GetDestination()->GetNrOfRecipients(); i++)
        {
            MPI_Send(Buffer->GetBase(), Buffer->GetPosition(), MPI_PACKED, 
                     (int) Recipients[i], MPI_DEFAULT_MSG_TYPE, MPI_COMM_WORLD);
        }
        break;

    default:
        MPI_Send(Buffer->GetBase(), Buffer->GetPosition(), MPI_PACKED, 
                 (int) GetDestination()->GetRecipient(), 
                 MPI_DEFAULT_MSG_TYPE, MPI_COMM_WORLD);
        break;
    }
    return TRUE;
}

// =================================================================================== 

TString
TMPIMessage::GetClassName(void) const

{
    return "TMPIMessage";
}

// =================================================================================== 

TString
TMPIMessage::GetInstanceName(void) const

{
    char Line[256];

    sprintf(Line, "%s", (char *) ParentClass::GetInstanceName());
    return Line;
}

// =================================================================================== 

void
TMPIMessage::_Init(void)

{
    Buffer = 0;
}

// =================================================================================== 

bool
TMPIMessage::Select(void)

{
    return TRUE;
}

// =================================================================================== 
#undef ParentClass
