/*      file     : mpi/TMPIMessageControl.h
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TMPIMessageControl.h,v 1.12 2005/05/01 14:57:58 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TMPIMESSAGECONTROL_H
#define TMPIMESSAGECONTROL_H

// ***INCLUDES************************************************************************ 

#include <TString.h>

#include <ParallelTypes.h>
#include <TCommunicationControl.h>

#include "Mpi.h"

// ***FORWARDS************************************************************************

class TMPIBuffer;

// ***CLASSES************************************************************************* 

class TMPIMessageControl : public TCommunicationControl

{
public:

    TMPIMessageControl(void);
    virtual ~TMPIMessageControl(void);

private:

    virtual bool
        Activate(void);
    bool
        GetBuffer(TMPIBuffer *&Buffer);
    virtual long
        GetCapabilities(void) const;
    virtual TString
        GetClassName(void) const;
    virtual TBaseMessage *
        GetNewMessage(TMessageID theID, TCPNID theSender, 
                      TDestination *theDestination, TMessageTag theTag,
                      TMessageType theType, TMessageFlags theFlags);
    void
        _Init(void);
    virtual bool
        IsBlockable(void) const;
    void
        LoadBaseTables(void);
    virtual bool
        MessageIsPending(void) const;
    virtual bool
        ReceiveMessage(TBaseMessage *&theMessage, long TimeOut);
    virtual bool
        ReduceLinearArray(const void* SendBuffer, void* ReceiveBuffer, 
                          int Count, TBaseType BaseType, 
                          TBaseOperation Operation, TCPNID Root);
    virtual bool
        SetEncoding(TEncodingMode theEncoding);

    int          BufferSize;
    int          MaxBuffer;
    MPI_Datatype BaseTypes[BaseTypeCount];
    MPI_Op       BaseOperations[BaseOperationCount];
};

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
