/*      file     : mpi/TMPIParallelControl.C
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TMPIParallelControl.C,v 1.9 2005/05/01 14:57:58 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <stdlib.h>
#include <stdio.h>

#include <Error.h>
#include <TEnvironment.h>

#include <Mpi.h>
#include <ParallelTypes.h>
#include <TCPNInfo.h>
#include <THostInfo.h>
#include "TMPIParallelControl.h"

// ***DEFINES*************************************************************************

#define MPI_MASTER_RANK   0

#define MPI_PARALLEL_CONTROL_PREFIX "MPI."

// ***METHODS************************************************************************* 

#ifdef NO_INLINE
#endif

// =================================================================================== 
#define ParentClass TParallelControl
// =================================================================================== 

TMPIParallelControl::TMPIParallelControl(int *pargc, char **pargv[])

{
    MPI_Init(pargc, pargv);
    _Init();
}

// =================================================================================== 

TMPIParallelControl::~TMPIParallelControl(void)

{
    Disenroll();
    DeleteObject(MpiMutable);
}

// =================================================================================== 

bool
TMPIParallelControl::Activate(void)

{
    return Enroll();
}

/* =================================================================================== */

bool
TMPIParallelControl::CheckCPNInfoList(void) const

{
    int       NumProcs;
    int       MyRank;
    TCPNInfo *CPNInfo;
    char      Name[32];
    int       i;
    char      Prefix[32];

    if (Mutable->CPNInfoList == 0)
    {
        if (!CheckHostInfoList())
            return FALSE;
        MPI_Comm_size(MPI_COMM_WORLD, &NumProcs);
        MPI_Comm_rank(MPI_COMM_WORLD, &MyRank);
        Mutable->CPNInfoList = new TCPNInfoList;
        for (i = 0; i < NumProcs; i++)
        {
            if ((CPNInfo = GetCPNInfo(Mutable->LocalHostInfo, i)) == 0)
            {
                Error("cannot create CPN Info");
                return FALSE;
            }
            sprintf(Name, "rank %d", i);
            CPNInfo->Name = Name;
            if (i == MyRank)
            {
                Mutable->LocalCPNInfo = CPNInfo;
                Environment->SetProcessName(Mutable->LocalCPNInfo->Name);
            }
            Mutable->CPNInfoList->Append(CPNInfo);          
        }
        sprintf(Prefix, "%d:", MyRank);
        Environment->SetPrintPrefix(Prefix);
    }
    return TRUE;
}

/* =================================================================================== */

bool
TMPIParallelControl::CheckHostInfoList(void) const

{
    int  MyRank;
    char HostName[MPI_MAX_PROCESSOR_NAME];
    int  NameLen;

    if (Mutable->HostInfoList == 0)
    {
        MPI_Get_processor_name(HostName, &NameLen);
        MPI_Comm_rank(MPI_COMM_WORLD, &MyRank);

        Mutable->HostInfoList = new THostInfoList;
        Mutable->LocalHostInfo = new THostInfo(HostName);
        Environment->SetHostName(Mutable->LocalHostInfo->GetShortName());
        
        Mutable->LocalHostInfo->ID = MyRank;
#ifdef DEBUG
        Mutable->LocalHostInfo->DebugMask = Debug.GetMask();
#endif
        Mutable->HostInfoList->Append(Mutable->LocalHostInfo);
    }
    return TRUE;
}

/* =================================================================================== */

bool
TMPIParallelControl::Disenroll(void)

{
    if (IsEnrolled())
    {
        Print("Disenroll from MPI");
        MPI_Finalize();
        if (!ParentClass::Disenroll())
            return FALSE;
    }
    return TRUE;
}

// =================================================================================== 

bool
TMPIParallelControl::Enroll(void)

{
    const char *ArchName;

    if (!IsEnrolled())
    {
        if (!ParentClass::Enroll())
            return FALSE;
        Print("Enroll into MPI");
        if ((ArchName = getenv("MPI_ARCH")))
        {
            char Arch[32];
            
            sprintf(Arch, "MPI_%s", ArchName);
            Environment->SetArchitecture(Arch);
        }
    }
    return TRUE;
}

// =================================================================================== 

TString
TMPIParallelControl::GetClassName(void) const

{
    return "TMPIParallelControl";
}

// =================================================================================== 

int
TMPIParallelControl::GetMaxNrOfCPN(void) const

{
    int NumProcs;
    
    MPI_Comm_size(MPI_COMM_WORLD, &NumProcs);
    return NumProcs;
}

// =================================================================================== 

TCPNID
TMPIParallelControl::GetParentCPNID(void) const

{
    return MPI_MASTER_RANK;
}

// ===================================================================================

void
TMPIParallelControl::_Init(void)

{
    MpiMutable = new TMpiMutable;
}

// ===================================================================================

bool
TMPIParallelControl::IsMaster(void) const

{
    int MyRank;

    MPI_Comm_rank(MPI_COMM_WORLD, &MyRank);
    return  MyRank == MPI_MASTER_RANK;
}

// ===================================================================================
#undef ParentClass
