/*      file     : partools/TObjectTimeStat.C
        project  : PAMINA III
        author   : Marcus Rickert
        RCS      : $Id: TObjectTimeStat.C,v 1.4 2005/04/01 15:11:25 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <TBaseStream.h>
#include <TObjectTimer.h>
#include <TTransferInfo.h>

#include <TObjectTimeStat.h>

// ***METHODS************************************************************************* 

#ifdef NO_INLINE
#endif

// =================================================================================== 

// =================================================================================== 
#define ParentClass TStatObject
// =================================================================================== 

TObjectTimeStat::TObjectTimeStat(TClassInfo *theClassInfo, TParallelID theParallelID,
                                 double theTime) :
    
ParentClass(theClassInfo, theParallelID)

{
    Time = theTime;
}

// =================================================================================== 

bool
TObjectTimeStat::Accumulate(const TStatObject *StatObject)

{
    const TObjectTimeStat *Stat = (const TObjectTimeStat *) StatObject;

    Time += Stat->Time;
    return ParentClass::Accumulate(StatObject);
}

// =================================================================================== 

void
TObjectTimeStat::Collect(TObjectTimer *Timer)

{
    Time = Timer->GetTime();
}

/* =================================================================================== */

bool
TObjectTimeStat::Decode(TBaseStream *aStream, TTransferInfo *TransferInfo)

{
    return (ParentClass::Decode(aStream, TransferInfo) AND
            aStream->Decode(Time));
}

/* =================================================================================== */

bool
TObjectTimeStat::Encode(TBaseStream *aStream, TTransferInfo *TransferInfo) const

{
    LongID ID;

    switch ((TransferInfo == 0)? TM_ConstData : TransferInfo->TransferMode)
    {
    case TM_ConstData:
    case TM_VarData:
        return (ParentClass::Encode(aStream, TransferInfo) AND
                aStream->Encode(Time));

    case TM_AsciiData:
        ID = GetParallelID();
        return (aStream->Encode(ID) AND
                aStream->Encode(Time) AND
                aStream->Write("\n"));

    default:
        ;
    }
    return TRUE;
}

/* =================================================================================== */

TString
TObjectTimeStat::GetClassName(void) const

{
    return "TObjectTimeStat";
}

// =================================================================================== 

double
TObjectTimeStat::GetLoad(TCPNID LocalCPNID, TInfoID AdjacenceInfoID, TInfoID LocationInfoID)

{
    return Time;
}

// =================================================================================== 
#undef ParentClass
