/*      file     : parbalance/TTransferLoad.C
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TTransferLoadInfo.C,v 1.4 2005/03/25 11:06:49 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <stdio.h>

#include "TTransferLoadInfo.h"

// ***GLOBAL VARIABLES****************************************************************  

TClassInfo *TTransferLoadInfo::ClassInfo = 0;

// ***METHODS************************************************************************* 

#ifdef NO_INLINE
#  include "TTransferLoadInfo.i.C"
#endif

// =================================================================================== 
#define ParentClass TParallelListObject
// =================================================================================== 

TTransferLoadInfo::TTransferLoadInfo(TParallelID theParallelID) :

ParentClass(ClassInfo, theParallelID)
    
{
    MigrationContext = 0;
    Reset();
}

// =================================================================================== 

void
TTransferLoadInfo::Acknowledge(void)

{
    if (NrOfMissingAcks == 0)
    {
        Warning("received superfluous acknowledge");
        return;
    }
    NrOfMissingAcks--;
}

// =================================================================================== 

TString
TTransferLoadInfo::GetClassName(void) const

{
    return "TTransferLoadInfo";
}

// =================================================================================== 

TString
TTransferLoadInfo::GetInstanceName(void) const

{
    char Line[256];

    switch (Mode)
    {
    case NoTransfer:
    case TransferSelected:
        
        if (TransferLoad >= 0.0)
            sprintf(Line, "time-step %s transfer %s%f to %lX", 
                    (char *) CollectTimeStep.GetInstanceName(), 
                    (Mode == TransferSelected)? "(selected) ":"",
                    TransferLoad, GetParallelID());
        else
            sprintf(Line, "time-step %s receive %f from %lX", 
                    (char *) CollectTimeStep.GetInstanceName(), 
                    TransferLoad, GetParallelID());
        break;

    case WaitForAcknowledge:

        sprintf(Line, "transfer %f to %lX (scheduled for time-step %s, missing acks:%d)", 
                TransferLoad, GetParallelID(),
                (char *) TransferTimeStep.GetInstanceName(), NrOfMissingAcks);
        break;
        
    case WaitUntilCPNReady:

        sprintf(Line, "transfer %f to %lX (scheduled for time-step %s, not ready:%d)", 
                TransferLoad, GetParallelID(),
                (char *) TransferTimeStep.GetInstanceName(), NrOfNonReadyCPN);
        break;
        
    case SendLoad:

        sprintf(Line, "transfer %f to %lX (in time-step %s, active)", 
                TransferLoad, GetParallelID(),
                (char *) TransferTimeStep.GetInstanceName());
        break;
        
    case WaitForConfirm:

        sprintf(Line, "wait for confirmation for receive from %lX "
                "(scheduled for time-step %s)", 
                GetParallelID(), (char *) TransferTimeStep.GetInstanceName());
        break;

    case ReceiveLoad:

        sprintf(Line, "receive from %lX (scheduled for time-step %s)", 
                GetParallelID(), (char *) TransferTimeStep.GetInstanceName());
        break;
    }   
    return Line;
}

// =================================================================================== 

void
TTransferLoadInfo::Ready(void)

{
    if (NrOfNonReadyCPN == 0)
    {   
        Warning("received superfluous ready event");
        return;
    }
    NrOfNonReadyCPN--;
}

// =================================================================================== 

void
TTransferLoadInfo::Reset(void)

{
    TransferLoad = 0.0;
    Mode = NoTransfer;
    DeleteObject(MigrationContext);
}

// =================================================================================== 
#undef ParentClass





