/*      file     : parstat/TSamplingDescription.h
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TSamplingDescription.h,v 1.7 2005/03/25 11:08:01 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TSAMPLINGDESCRIPTION_H
#define TSAMPLINGDESCRIPTION_H

// ***INCLUDES************************************************************************ 

#include <TCPNInfo.h>
#include <ParStatTypes.h>
#include <TDLList.h>
#include <TDuration.h>
#include <TLocation.h>
#include <TParallelListObject.h>

// ***DEFINES*************************************************************************

#define SF_AGGREGATE     0x0001
#define SF_NEIGHBORS     0x0002
#define SF_MERGE         0x0004

#define SF_DEFAULT_FLAGS (SF_AGGREGATE)

// ***FORWARDS************************************************************************

class TStatisticsHandler;

#include <TStatObject.forward.h>

// ***CLASSES************************************************************************* 

class TSamplingDescription : public TParallelListObject

{
public:

    TSamplingDescription(TParallelID theSamplingID);
    TSamplingDescription(TClassID theClassID, int theStatFlags,
                         TCPNID theDestination, TParallelID theSamplingID);

    static TClassInfo *ClassInfo;

    virtual bool
    Decode(TBaseStream *Stream, TTransferInfo *);
    
    virtual bool
    Encode(TBaseStream *Stream, TTransferInfo *) const;

    virtual TString
    GetClassName(void) const;

    inline TCPNID
    GetDestination(void) const;

    inline TCPNInfoList *
    GetDestList(void) const;

    inline const TRectangle &
    GetFrame(void) const;

    inline TStatObjectList *
    GetObjectList(void);

    inline TClassID
    GetStatObjectClassID(void) const;

    inline TTimeStep &
    GetTimeStep(void);

    virtual TString
    GetInstanceName(void) const;

    inline bool
    IsAggregationStat(void) const;

    inline bool
    IsMergeStat(void) const;

    inline bool
    IsNeighborStat(void) const;

    inline bool
    NeedsCollecting(const TTimeStep &CurrentTimeStep) const;

    inline bool
    NeedsReset(void) const;

    inline bool
    NeedsSampling(const TTimeStep &CurrentTimeStep) const;

    inline void
    SetDestList(TCPNInfoList *theDestList);

    inline void
    SetDuration(TDuration &theDuration);

    inline void
    SetFrame(const TRectangle &theFrame);

    inline void
    SetCollectInterval(int theInterval);

    inline void
    SetSampleInterval(int theInterval);

    inline void
    SetTimeStep(const TTimeStep &theTimeStep);

private:

    void
    _Init(void);

    TDuration       Duration;
    TRectangle      Frame;
    TClassID        StatObjectClassID;
    TCPNID          Destination;
    TCPNInfoList   *DestList;
    TStatObjectList ObjectList;
    int             SampleInterval;
    int             CollectInterval;
    int             SubTimeStep;
    TTimeStep       TimeStep;
    int             StatFlags;
};

// ***TYPES*************************************************************************** 

typedef TDLList< TSamplingDescription > TSamplingDescriptionList;
typedef TDLListEnum< TSamplingDescription > TSamplingDescriptionEnum;

// ***INLINE METHODS******************************************************************

#ifndef NO_INLINE
#  include "TSamplingDescription.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif

