/*      file     : partools/TClassInfo.h
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TClassInfo.h,v 1.12 2005/03/25 11:08:14 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TCLASSINFO_H
#define TCLASSINFO_H

// ***INCLUDES************************************************************************ 

#include <TString.h>
#include <TIDContainerEntry.h>
#include <TABTree.h>
#include <TDLList.h>
#include <TBaseObject.h>
#include <TObjectHandler.h>
#include <ParToolsTypes.h>

// ***DEFINES************************************************************************* 

#define CI_FLAG_TABLE             1
#define CI_FLAG_CONTAINER_CLASS   2

// ***FORWARDS************************************************************************

class TParallelObject;
class TContainerObject;

// #include "TObjectHandler.forward.h"
#include "TParallelObject.forward.h"

// ***TYPES*************************************************************************** 


// ***CLASSES************************************************************************* 

/** Information about a class. */

class TClassInfo : public TBaseObject

{
    friend class TObjectManager;
    
public:

    TClassInfo(int theFlags, const char *theContainerClassName, const char *theClassName, 
               TObjectHandler *theObjectHandler);
    TClassInfo(const char *theClassName = 0);
    virtual ~TClassInfo(void);

    void
    Delete(const TParallelObject *Object) const;

    inline TObjectHandlerList &
    GetActionHandlers(void);

    virtual TString
    GetClassName(void) const;

    inline TClassInfo *
    GetContainerClass(void) const;

    inline TClassID
    GetID(void) const;

    inline TClassInfo *
    GetInfoClassInfo(TInfoID theInfoID) const;

    inline TInfoID
    GetInfoID(void) const;

    inline TInfoMask
    GetInfoMask(void) const;

    virtual TString
    GetInstanceName(void) const;

    inline char *
    GetName(void);

    inline int
    GetNrOfInfoClasses(void) const;

    inline TObjectHandler *
    GetObjectHandler(void) const;

    inline TParallelObject *
    GetObjectPtr(TParallelID theParallelID, bool Warn = FALSE) const;

    inline TParallelObjectTable *
    GetTable(void) const;

    inline bool
    HasTable(void) const;

    void
    Insert(const TParallelObject *Object) const;
    
    inline bool
    IsContainer(void) const;
    
    inline bool
    RegisterActionHandler(TObjectHandler *theActionHandler);

private:
    
    virtual int
    Compare(TBaseObject *OtherObject, int CompareCode = 0) const;

    TParallelObjectTable *Table;

    int                 ClassFlags;
    TClassInfo         *ContainerClass;
    int                 NrOfInfoClasses;
    TClassInfo        **InfoClasses;
    TString             Classname;
    const char         *ContainerClassname;
    TClassID            ClassID;
    TInfoID             InfoID;
    TObjectHandler     *ObjectHandler;
    TObjectHandlerList  ActionHandlers;
};

// ***INLINE METHODS******************************************************************

#ifndef NO_INLINE
#  include "TClassInfo.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
