/*      file     : partools/TEdge.h
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TEdge.h,v 1.12 2005/03/25 11:08:14 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TEDGE_H
#define TEDGE_H

// ***INCLUDES************************************************************************ 

#include <TLocation.h>
#include <TContainerObject.h>

#include <TAdjacenceInfo.h>

// ***FORWARDS************************************************************************

class TClassInfo;
class TDistributedRange;
class TNode;

// ***CLASSES************************************************************************* 

/** Base class to the simulation edge class.
  */

class TEdge : public TContainerObject

{
public:

    TEdge(TClassInfo *theClassInfo, TParallelID theParallelID,
          TInfoID theDefaultAdjacenceInfoID);

    virtual double
    GetActiveFraction(TCPNID CONID) const;

    void
    GetActiveRange(TCPNID CPNID, TDistributedRange *Range) const;

    inline TNode *
    GetFromNode(TInfoID AdjacenceInfoID = INVALID) const;
    
    TCPNID
    GetFromNodeCPNID(void) const;

    inline TNode *
    GetOtherNode(TNode *aNode, TInfoID AdjacenceInfoID  = INVALID) const;

    TCPNID
    GetToNodeCPNID(void) const;
    
    inline TParallelID
    GetFromNodeID(TInfoID AdjacenceInfoID = INVALID) const;
    
    TCoordinate
    GetLength(TInfoID AdjacenceInfoID, TInfoID LocationInfoID = INVALID);
    
    static inline TEdge *
    GetPtr(TParallelID ID, bool Warn = FALSE);

    inline TNode *
    GetToNode(TInfoID AdjacenceInfoID = INVALID) const;

    inline TParallelID
    GetToNodeID(TInfoID AdjacenceInfoID = INVALID) const;

    static TEdge *
    LoadInstance(TParallelID EdgeID, TInfoMask LocalInfoMask = 0, bool LoadNodes = FALSE);

    static TClassInfo *ClassInfo;

private:

    bool
    Decode(TBaseStream *Stream, TTransferInfo *TransferInfo);

    virtual bool
    DisposeNonLocalPortion(TCPNID);

    virtual bool
    Encode(TBaseStream *Stream, TTransferInfo *TransferInfo) const;

    virtual TString
    GetClassName(void) const;

    virtual bool
    GetRequiredContainers(int &NrOfRefs, TContainerReference *);

    TInfoID DefaultAdjacenceInfoID;
};

// ***TYPES*************************************************************************** 

typedef TDLList< TEdge > TEdgeList;
typedef TDLListEnum < TEdge > TEdgeEnum;

// ***INLINE METHODS******************************************************************

#ifndef NO_INLINE
#  include "TEdge.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
