/*      file     : pvm/TPVMMessageControl.C
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TPVMMessageControl.C,v 1.15 2005/03/25 11:08:26 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <stdlib.h>
#include <Pvm.h>

#include <PVMConfig.h>

#include <TEnvironment.h>
#include <TConfig.h>

#include <TParallel.h>
#include <TCPNInfo.h>
#include "TConfiguration.h"
#include "TPVMMessage.h"
#include "TPVMMessageControl.h"

// ***CONSTANTS***********************************************************************

char *
PvmEncodings[] =

{
    "DataDefault (XDR)",
    "DataRaw",
    "DataInPlace",
    "DataFoo?"
};

// ***METHODS************************************************************************* 

// =================================================================================== 
#define ParentClass TCommunicationControl
// =================================================================================== 

TPVMMessageControl::TPVMMessageControl(void)

{
    _Init();
}

// =================================================================================== 

long
TPVMMessageControl::GetCapabilities(void) const

{
    return CC_SEND_MESSAGE;
}

// =================================================================================== 

TString
TPVMMessageControl::GetClassName(void) const

{
    return "TPVMMessageControl";
}

// =================================================================================== 

TBaseMessage *
TPVMMessageControl::GetNewMessage(TMessageID theID, TCPNID theSender,
                                  TDestination *theDestination,
                                  TMessageTag theTag, TMessageType theType,
                                  TMessageFlags theFlags)

{
    return new TPVMMessage(this, theID, theSender, theDestination, theTag, theType, 
                           theFlags, Encoding);
}

// =================================================================================== 

void
TPVMMessageControl::_Init(void)

{
    TConfiguration *C = Environment->GetConfiguration();

    C->Set(TConfigEntry(PVM_GROUP, PVM_NAME_ENCODING, (long) PVM_NAME_ENCODING_DEF));
}

// =================================================================================== 

bool
TPVMMessageControl::IsBlockable(void) const

{
    return TRUE;
}

// =================================================================================== 

bool
TPVMMessageControl::MessageIsPending(void) const

{
    int BufferID;

    if ((BufferID = pvm_probe(PVM_ANY_HOST, PVM_ANY_TYPE)) < 0)
    {
        Warning("pvm_probe returns %d", BufferID);
        return FALSE;
    }
    if (BufferID != 0)
        debug(Print(DM_MSG_CONTROL, "message pending with buffer ID %d", BufferID));
    return (BufferID != 0);
}

// =================================================================================== 

bool
TPVMMessageControl::ReceiveMessage(TBaseMessage *&theMessage, long Milliseconds)

{
    int BufferID;
    int PvmError;
    int Length;
    int Type;
    int Sender;

    theMessage = 0;
    if (Milliseconds > 0)
    {
        struct timeval TimeOut;

        TimeOut.tv_sec = (int) Milliseconds / 1000;
        TimeOut.tv_usec = 1000 * (Milliseconds % 1000);
        if ((BufferID = pvm_trecv(PVM_ANY_HOST, PVM_ANY_TYPE, &TimeOut)) < 0) 
        {
            Error("ReceiveMessage: pvm_recv returns %i", BufferID);
            return FALSE;
        }
        if (BufferID == 0)
        {
            Error("ReceiveMessage: time-out after %ld milliseconds", Milliseconds);
            return FALSE;
        }
    }
    else
        if ((BufferID = pvm_recv(PVM_ANY_HOST, PVM_ANY_TYPE)) < 0) 
        {
            Error("ReceiveMessage: pvm_recv returns %i", BufferID);
            return FALSE;
        }
    if ((PvmError = pvm_bufinfo(BufferID, &Length, &Type, &Sender)) < 0)
    {
        Error("ReceiveMessage: pvm_bufinfo returns %i", PvmError);
        return TRUE;
    }
    debug(Print(DM_MSG_CONTROL, 
                "received PVM message (BufferID:%d Length:%d Type:%d) from %X",
                BufferID, Length, Type, Sender));
    switch (Type)
    {
    case PVM_DEFAULT_MSGTYPE:
        return ReceiveRegularMessage(BufferID, Sender, theMessage);

    case PVM_NOTIFY_ADD_HOST_MSGTYPE:
        return FALSE;    //   !!!!! TEMP !!!!!

    case PVM_NOTIFY_DELETE_HOST_MSGTYPE:
        return FALSE;    //   !!!!! TEMP !!!!!

    default:
        Error("ReceiveMessage: unknown PVM message type %d from %X", Type, Sender);
        return FALSE;
    }
#ifndef COMPLETE_SWITCH_FLOW
    return TRUE;
#endif
}

// =================================================================================== 

bool
TPVMMessageControl::ReceiveRegularMessage(int BufferID, TCPNID Sender,
                                          TBaseMessage *&theMessage)

{
    if ((theMessage = new TPVMMessage(this, BufferID, Sender)) == 0)
    {
        Error("ReceiveRegularMessage: cannot instantiate TPVMMessage");
        return FALSE;
    }
    if (!theMessage->BeginReceive())
        return FALSE;
    debug(Print(DM_MSG_CONTROL,"ReceiveRegularMessage: %s", 
                (char *) theMessage->GetInstanceName()));
    return TRUE;
}

/* =================================================================================== */

bool
TPVMMessageControl::SetEncoding(TEncodingMode theEncoding)

{
    switch(theEncoding)
    {
    case EncodingRaw:
        Encoding = PvmDataRaw;
        break;

    case EncodingXDR:
        Encoding = PvmDataDefault;
        break;

    default:
        Error("illegal encoding mode %d", (int) theEncoding);
        return FALSE;
    }
    return TRUE;
}

/* =================================================================================== */
/*
void
TPVMMessageControl::WriteSignature(void) const

{
    SignatureHeader(GetClassName());
    SignatureLine("PvmEncoding    = %s", PvmEncodings[Encoding]);
}
*/
// =================================================================================== 
#undef ParentClass
