/*      file     : pvm/TPVMParallelControl.h
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TPVMParallelControl.h,v 1.11 2005/03/25 11:08:26 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TPVMPARALLELCONTROL_H
#define TPVMPARALLELCONTROL_H

// ***INCLUDES************************************************************************ 

#include <TDLList.h>
#include <ParallelTypes.h>
#include <TParallelControl.h>

// ***FORWARDS************************************************************************

class TCPNInfo;
class THostInfo;

// ***TYPES*************************************************************************** 

struct TPvmMutable

{
    struct pvmhostinfo *PvmHostInfos;
    int                 PvmNrOfHosts;
    int                 PvmNrOfArchs;    
};

// ***CLASSES************************************************************************* 

class TPVMParallelControl : public TParallelControl

{
public:

    TPVMParallelControl(void);
    virtual ~TPVMParallelControl(void);

protected:

    virtual bool
        Activate(void);
    virtual bool
        AddCPN(THostInfo *aHostInfo, int &NrOfInstances);
    virtual bool
        Disenroll(void);
    virtual bool
        Enroll(void);
    virtual TString
        GetClassName(void) const;
    virtual bool
        IsMaster(void) const;

private:

    virtual bool
        CheckCPNInfoList(void) const;
    virtual bool
        CheckHostInfoList(void) const;
    bool
        CPNonHost(TCPNID aCPNID, THostID aHostID) const;
    bool
        FakeMasterHostInfo(struct pvmhostinfo *aHostInfo) const;
    THostInfo *
        GetHostInfo(const char *Hostname) const;
    virtual TCPNID
        GetParentCPNID(void) const;
    bool
        GetPvmHostInfos(void) const;
    void
        _Init(void);
    bool
        MarkTopologyChanged(void);
    bool
        ReadHostfile(void);

    TCPNID              LocalCPNID;
    TCPNID              ParentCPNID;
    int                 RoutingMode;
    TString             HostFilename;
    TDLList<THostInfo> *InactiveHosts;
    TPvmMutable        *PvmMutable;
};

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
